/*!
 * @file
 * @brief Implementation of error handling routines for ECE 486
 * @author Don Hummels
 * @date Jan, 2013
 */

/*
	Sets up a global error (circular) buffer. When flagerror() is called, a value
	corresponding to that error is pushed into the buffer. The header file contains
	the list of #define's that match error numbers with their errors. When an error
	occurs, the red LED on port D, pin 14 will light up, and remain lit.
*/

#include "stm32l4xx_hal.h"
#include "stm32l476g_discovery.h"

#include "err486.h"
#include "init486.h"
#include "stm32l476g_discovery_glass_lcd.h"
#include <stdio.h>

static void print_error(int index);

extern enum Num_Channels_Out Output_Configuration;
extern enum Num_Channels_In Input_Configuration;

int errorbuf[ERRORBUFLEN];
int first_error=0;
int erroridx = 0;

/*
 *	initerror() is called once to set up the global error buffer
 */
void initerror()
{
  int i;
  
  for (i=0; i<ERRORBUFLEN; i++) errorbuf[i] = 0;
  
  //turn off our error indicating LED
  BSP_LED_Off(ERROR_LED);
}

/*
	flagerror() is called when an error occurs.
	errorcode is written to the error buffer.
*/
void flagerror(int errorcode)
{
  char err_str[8];
  
  // Display the first error on the LCD if available
  if (first_error == 0) {
    first_error = errorcode;
    sprintf(err_str, "ERR %2d", errorcode);
    BSP_LCD_GLASS_DisplayString((uint8_t*)err_str);
    print_error(errorcode);
  }
  
  //turn on our error indicating LED
  BSP_LED_On(ERROR_LED);
  
  
  // Store an array of the most recent errors for examination by a debugger.
  errorbuf[erroridx] = errorcode;
  erroridx++;
  if (erroridx == ERRORBUFLEN) erroridx = 0;
}

static void print_error(int index){

    char* error;
    
    if(index == 2)		error = "SAMPLE_OVERRUN";
    else if(index == 3)		error = "MEMORY_ALLOCATION_ERROR";
    else if(index == 4)		error =	"DAC_CONFIG_ERROR";
    else if(index == 5)		error =	"ADC_CONFIG_ERROR";
    else if(index == 6)		error =	"SETBLOCKSIZE_ERROR";
    else if(index == 7)		error = "INVALID_MIC_SAMPLE_RATE";
    else if(index == 8)		error = "CLOCK_CONFIG_ERROR";
    else if(index == 13)	error = "DEBUG_ERROR";
    else error = "UNKNOWN";
    
    printf("\n*** ERROR %d ***: %s\n",index, error);
}
