/*!
 * @file
 * 
 * @brief USART Initialization to stream printf() output back to the ST-LINK
 * 
 * @author Hummels... really stolen from Sheaff.  
 * 
 * @date Jan  2016
 * 
 * ST-LINK V2 provides a serial port to the host computer, which is accessed
 * through a serial connector configured here...
 */

#include "stm32l4xx_hal.h"
#include "usart.h"

UART_HandleTypeDef UsartHandle;

// Setup L4 serial port to connect the STM debug IC
// Standard settings - 115200 8N1
// Be sure to get the updated syscalls.c file to redirect write to the USART
void usart_init(void)
{
	UsartHandle.Instance = USART2;
	UsartHandle.Init.BaudRate = 115200;
	UsartHandle.Init.WordLength = UART_WORDLENGTH_8B;
	UsartHandle.Init.StopBits = UART_STOPBITS_1;
	UsartHandle.Init.Parity = UART_PARITY_NONE;
	UsartHandle.Init.HwFlowCtl = UART_HWCONTROL_NONE;
	UsartHandle.Init.Mode = UART_MODE_TX_RX;
	UsartHandle.Init.OverSampling = UART_OVERSAMPLING_16;
	HAL_UART_Init(&UsartHandle);

	return;
}


void HAL_UART_MspInit(UART_HandleTypeDef * huart)
{
	GPIO_InitTypeDef GPIO_InitStruct;

	__HAL_RCC_GPIOD_CLK_ENABLE();
	__HAL_RCC_USART2_CLK_ENABLE();

	GPIO_InitStruct.Pin = GPIO_PIN_5;
	GPIO_InitStruct.Mode = GPIO_MODE_AF_PP;
	GPIO_InitStruct.Pull = GPIO_PULLUP;
	GPIO_InitStruct.Speed = GPIO_SPEED_FREQ_VERY_HIGH;
	GPIO_InitStruct.Alternate = GPIO_AF7_USART2;
	HAL_GPIO_Init(GPIOD, &GPIO_InitStruct);

	/* UART RX GPIO pin configuration  */
	GPIO_InitStruct.Pin = GPIO_PIN_6;
	GPIO_InitStruct.Alternate = GPIO_AF7_USART2;

	HAL_GPIO_Init(GPIOD, &GPIO_InitStruct);
}
